# AUTHOR: JOSHUA KIM

# PURPOSE: Takes the consumer dataset, selects those who show up in every 8 week interval,
# and then creates the master consumer panel. 

# INPUTS:
# 1. consumers.csv 

# OUTPUTS:

rm(list=ls())

# -------------------------------
#        FUNCTIONS

getDtaName = function(i){
  # Given an integer i, returns a string called
  # 'wi.dta'
  
  dta.file = paste('w', i, '.dta', sep = '')
  
  return(dta.file)
}

sampleConsumers = function(dt){
  # Given a dt, will apply several criteria to the data and return a list of 
  # N consumer IDs that we use to create the panel data.
  
  # Want consumers who show up at least once every 8 week period.
  min.wk = min(dt$week)
  max.wk = max(dt$week)
  
  wk.ranges = seq(min.wk, max.wk, by = 8)
  
  customer.ids = as.set(dt[, rut_doc])
  for (i in 1:(length(wk.ranges) - 1)){
    # For each i, get the list of customers who have at least one account
    # between week i and week i + 8
    i.period = as.set(dt[wk.ranges[i] <= week & week < wk.ranges[i+1], rut_doc])
    
    # Then intersect with the current list of customers who did this in all previous
    # weeks.
    customer.ids = set_intersection(customer.ids, i.period)
  }

  return(customer.ids)
}

cleanDTA = function(raw.dt, customer.keys){
  # Given a weekly dta file, cleans it.
  
  # Only include the data for customers we care about
  dt = raw.dt[rut_doc %in% customer.keys]
  
  # Get rid of useless variables
  dt = dt[, (drop.vars) := NULL]
  
  # Get the UPC ID so we can merge to products
  dt = dt[, upc_id := substr(bar_code, 1, nchar(bar_code) - 1)]
  dt = merge(dt, products, how = 'inner', on = 'upc_id')
  
  # Only include food
  dt = dt[food == 1,]
  dt = dt[, food := NULL]
  
  # Fix some wonky quantities
  dt = dt[quantity >-1 & quantity < 0, quantity := quantity * 1000]
  
  # Aggregate by date, upc_id, store_id, and the rut_doc
  dt = dt[, 
          c('sales_final', 'sales_raw', 'quantity') := list(sum(sales_final), sum(sales_raw), sum(quantity)), 
          by = c('date', 'upc_id', 'store_id', 'rut_doc')]
  
  return(dt)
}

cleanPanel = function(panel.dt){
  # Given the panel dataset, cleans it up.
  
  # Delete any negative observations
  panel.dt = panel.dt[(sales_final >= 0) &
                        (sales_raw >= 0) & 
                        (quantity >= 0), ]
  
  # Drop if a completely zero observation
  panel.dt = panel.dt[!((sales_final == 0) & (sales_raw == 0) & (quantity == 0))]
  return(panel.dt)
}

# ------------------------------
#         SET UP 

library(data.table)
library(readstata13)
library(sets)

user.name = Sys.info()[7]

consumer.dir = './00_files/02_intermediate/03_consumers'
nutrition.dir = './00_files/02_intermediate/02_nutrition'
dta.dir = './00_files/02_intermediate'


# ---------------------------
#         CONSUMERS

# Get consumer data
setwd(consumer.dir)
consumers = fread('01_consumers.csv')
selected_consumers = sampleConsumers(consumers)
print(length(selected_consumers))

# ---------------------------
# Get products_food data
setwd(nutrition.dir)
products = fread('02b_products_food.csv')
products = products[, .(upc_id, category, food)]
products[, upc_id := as.character(upc_id)]

# -------------------------

# DTA Files

setwd(dta.dir)

panel.dt = data.table()

# For each week, get the relevant data, clean it, and then add it to the main panel dataset.
NUM.WEEKS = 152

for (i in 1:NUM.WEEKS){
  file.name = getDtaName(i)
  raw.dta = data.table(read.dta13(file.name))
  
  dta = cleanDTA(raw.dta, selected_consumers)
  panel.dt = rbind(panel.dt, dta)
}

panel.dt = cleanPanel(panel.dt)

panel.dt = panel.dt[order(rut_doc, date, upc_id, store_id)]

# Write out the data

setwd(consumer.dir)
fwrite(panel.dt, file = '02_MasterConsumerPanelData.csv')

