/*******************************************************************************
AUTHOR: Nano Barahona
CREATION: Jun-2021
ACTION: Cleans cereal's consumer panel for model estimation with heterogeneities
SUMMARY:

********************************************************************************/

clear all
set more off
set scheme s1mono

******************************************************************************** 
** SET PATH AND LOCALS
******************************************************************************** 

if c(username)=="Nano"   		  global Repo "/Users/Nano/Dropbox/Research/FoodLabels/food_labels/04_replication"
else 							  global Repo "."

******************************************************************************** 
** LOAD DATA AND MERGE SCANNED AND STORE DATA
******************************************************************************** 

use "${Repo}/00_files/01_source/01_walmart/locales_deptos.dta" , clear
destring store_id , replace
tempfile temp
save `temp', replace


use "${Repo}/00_files/02_intermediate/01_panel/03_MasterConsumerPanelData_Cereals.dta", clear

merge n:1 upc_id using "${Repo}/00_files/02_intermediate/02_nutrition/04_cereals_nutrition.dta"
keep if _merge==3
keep if scannedL == 1

keep nutrition_id store_id sales_final sales_raw quantity sell_q week week8 rut_doc

merge n:1 store_id using `temp' 
keep if _merge==3
drop _merge
keep if strpos(Formato,"EXPRESS")>0 | strpos(Formato,"LIDER")>0

keep nutrition_id store_id sales_final sales_raw quantity sell_q week week8 rut_doc


merge n:1 rut_doc using  "${Repo}/00_files/02_intermediate/03_consumers/consumer_characteristics.dta"
keep if _merge==3
drop _merge
drop income_bins age_bins
gen age_bins = 1+ (age >= 35) + (age >= 45) + (age > 60)   
gen income_bins = 1 + (income>0.7)
gen individual_bins = income_bins


******************************************************************************** 
** IMPORT NUTRITION ID INFORMATION AND DEFINE PRODUCT FINELINES
******************************************************************************** 

do "${Repo}/02_build/02_model/aux/01_new_fineline.do"
keep if _merge==3
drop _merge

drop name brand Lcal Lsug Lsod Lfat cal_post cal_pre cal_late sug_post sug_pre sug_late sod_post sod_pre sod_late fat_post fat_pre fat_late healthy

******************************************************************************** 
** GET RID OF OUTLIERS
******************************************************************************** 

drop if new_fineline == "DILATADO"

gen outlier = 0

replace outlier = 1 if rut_doc==0
replace outlier = 1 if rut_doc > 25000000

egen unique_user_week = tag(week rut_doc)
egen unique_user = tag(rut_doc)

egen weekly_spending = sum(sales_final), by(week rut_doc)
egen max_spending = max(weekly_spending), by(rut_doc)

sum max_spending if unique_user == 1 & outlier == 0, det
local maxval = r(p99)
di `maxval'

count if max_spending>`maxval' & unique_user == 1
replace outlier = 1 if max_spending>`maxval'

save "${Repo}/00_files/03_data/temp/01_carealpanel.dta", replace

******************************************************************************** 
** CALCULATE GRAMS SOLD AND OUTSIDE OPTION MARKET SHARE
******************************************************************************** 

use "${Repo}/00_files/03_data/temp/01_carealpanel.dta", clear

replace outlier = 1 if census_toofar == 1
replace outlier = 1 if age < 18

drop if outlier == 1

egen store_week = tag(store_id week8)
egen customer_store_week = tag(week8 rut_doc store_id) 
egen customer_store = tag(rut_doc store_id) 
egen bin_store_week = tag(week8 individual_bins store_id)

egen num_cus8_cer_bins = total(customer_store_week), by(week8 store_id individual_bins)

destring sell_q, replace dpcomma
gen quantity_unit = quantity
replace quantity = quantity*sell_q

egen inside_quantity_bin = total(quantity), by(store_id week8 individual_bins)
egen inside_quantity = total(quantity), by(store_id week8)

gen pre = (week8<=6)

egen total_quant_store_bin = total(quantity) if pre==1, by(store_id individual_bins)
egen total_cus_store_bin = total(num_cus8_cer_bins*bin_store_week) if pre==1, by(store_id individual_bins)
gen temp_bin = total_quant_store_bin/total_cus_store_bin
egen portion_size_bin = max(temp_bin), by(store_id individual_bins)

egen total_quant_store = total(quantity) if pre==1, by(store_id)
egen total_cus_store = total(num_cus8_cer_bins*bin_store_week) if pre==1, by(store_id)
gen temp = total_quant_store/total_cus_store
egen portion_size = max(temp), by(store_id)

egen market_size_bin = total(customer_store), by(store_id individual_bins)
egen market_size = total(customer_store), by(store_id)

gen outside_opt_bin   = 1-inside_quantity_bin/(market_size_bin*portion_size_bin)
gen outside_opt 	  = 1-inside_quantity/(market_size*portion_size)

save "${Repo}/00_files/03_data/temp/02_carealpanel.dta", replace


******************************************************************************** 
** COLLAPSE DATA FOR SMALL BINS SAMPLE
******************************************************************************** 


use "${Repo}/00_files/03_data/temp/02_carealpanel.dta", replace


collapse (sum) quantity quantity_unit sales_final sales_raw (first) ///
market_size_bin market_size portion_size_bin portion_size outside_opt_bin outside_opt ///
income_bins  ,  by(store_id week8 nutrition_id individual_bins)


save "${Repo}/00_files/03_data/temp/03_carealpanel.dta", replace


******************************************************************************** 
** IMPORT NUTRITION ID INFORMATION AND DEFINE PRODUCT CHARACTERISTICS LARGE BINS
******************************************************************************** 

use "${Repo}/00_files/03_data/temp/03_carealpanel.dta", clear

do "${Repo}/02_build/02_model/aux/01_new_fineline.do"
ren new_fineline fineline_name 
keep if _merge==3
drop _merge

do "${Repo}/02_build/02_model/aux/02_code_ingredients.do"


merge n:1 store_id using "${Repo}/00_files/01_source/01_walmart/traits_raw.dta"
keep if _merge==3
drop _merge

merge n:1 nutrition_id using "${Repo}/00_files/02_intermediate/04_beliefs/02_cereals_beliefs_final"
keep if _merge==3
drop _merge brand_name fineline known_cal w_cal w_post_cal p_cal time_cal l_cal muse_cal obs_cal known_sug w_sug w_post_sug p_sug time_sug l_sug muse_sug obs_sug


******************************************************************************** 
** CONSTRUCT PRICE INSTRUMENT
******************************************************************************** 

global commodity "sugar corn cocoa rice wheat oat"

foreach com in $commodity {
	
preserve

import delimited "${Repo}/00_files/01_source/07_commodities/`com'-prices-historical-chart-data.csv", clear 

if inlist("`com'","sugar","wheat") {
	gen weekMonday = date(date, "YMD")
}
else {
	gen weekMonday = date(date, "MDY")
	ren closelast value
}

duplicates drop weekMonday, force

merge 1:1 weekMonday using "${Repo}/00_files/01_source/08_others/week2date.dta"

sort weekMonday

replace week = week[_n-1] if week[_n-1]!=. & week==.

drop if weekMonday>=21279 | weekMonday<=20209

gen week8 = floor((week-1)/8)+1

collapse (mean) value, by(week8)

ren value `com'_price

tempfile `com'_temp
save ``com'_temp', replace

restore

}

merge n:1 week using `sugar_temp'
drop _merge
merge n:1 week using `corn_temp'
drop _merge
merge n:1 week using `cocoa_temp'
drop _merge
merge n:1 week using `rice_temp'
drop _merge
merge n:1 week using `wheat_temp'
drop _merge
merge n:1 week using `oat_temp'
drop _merge


save "${Repo}/00_files/03_data/temp/04_carealpanel.dta", replace



******************************************************************************** 
** PREPARE FINAL VARIABLES FOR LARGE BIN SAMPLE
******************************************************************************** 

use "${Repo}/00_files/03_data/temp/04_carealpanel.dta", clear

drop if brand == "GO OATS-15404"

* define policy period
gen policy= floor((61-1)/8)+1         
gen penetration = floor((44-1)/8)+1
gen post = week>policy
* define identifiers
ren brand brand_name
ren week8 week
egen product = group(nutrition_id)
egen store = group(store_id)
egen fineline = group(fineline_name)
egen brand = group(brand_name)
egen market_bin = group(week store individual_bins)
egen market = group(week store)
* define conditional market shares
egen totq = sum(quantity), by(market_bin)
gen s_inside = quantity/totq
egen totqfineline = sum(quantity), by(market_bin fineline)
gen s_fineline = quantity/totqfineline
egen totqhealthy = sum(quantity), by(market_bin healthy)
gen s_healthy = quantity/totqhealthy
* define market shares
ren outside_opt_bin s0
gen s = quantity/(market_size_bin*portion_size_bin)
* define price
egen sales_final_bigmarket = sum(sales_final), by(market nutrition_id)
egen sales_raw_bigmarket = sum(sales_raw), by(market nutrition_id)
egen quantity_bigmarket = sum(quantity), by(market nutrition_id)
gen price_kilo = (sales_final_bigmarket/quantity_bigmarket)/650
gen p = price_kilo/10
gen price_raw = (sales_raw_bigmarket/quantity_bigmarket)/(650*10)
gen price_bins = (sales_final/quantity)/(650*10)
* define price instruments (including hausman instruments for robustness checks)
egen temps = sum(sales_final) , by(nutrition_id week cluster_walmart) // store_cluster
egen tempq = sum(quantity) , by(nutrition_id week cluster_walmart) // store_cluster
egen temps2 = sum(sales_final) , by(nutrition_id week ) // store_cluster
egen tempq2 = sum(quantity) , by(nutrition_id week ) // store_cluster
replace temps = temps-sales_final_bigmarket
replace tempq = tempq-quantity_bigmarket
replace temps2 = temps2-sales_final_bigmarket
replace tempq2 = tempq2-quantity_bigmarket
gen inst_haus = (temps/tempq)/(650*10)
gen temp_haus = (temps2/tempq2)/(650*10)
replace inst_hau = temp_haus if temps < 1
gen inst_dis = p/price_raw
gen inst_sug = sugar_price*sug_pre 
gen inst2_sug = sugar_price*(sug_pre>22.5)
foreach var in oat rice cocoa corn wheat {
gen inst_`var' = `var'_price*`var'
}
** construct external instruments for random effects **
reghdfe p inst_corn inst_wheat inst_oat, cl(market) absorb(product store week) resid
predict hat_p , xbd
egen fine_p_mean = mean(hat_p) , by(fineline market)
egen fine_p_p80 = pctile(hat_p) , by(fineline market) p(80)
egen fine_p_p20 = pctile(hat_p) , by(fineline market) p(20)
foreach varname in "fine_p_mean" "fine_p_p80" "fine_p_p20"  {
	forv i = 1/5 {
		gen temp`i' = `varname' if fineline==`i'
		egen hat_`varname'_`i' = max(temp`i'), by(market)
		drop temp`i'
	}
}
drop fine_p_*
* redefine variables
foreach var in sug cal fat sod {
gen `var' = `var'_pre*(1-post)+`var'_post*post
}
* define exempted products
gen exempted = (fineline_name == "AVENA" & nutrition_id != "826085")
* define package composition
gen package = quantity/quantity_unit

* redefine variables
drop product store fineline brand
egen product = group(nutrition_id)
egen store = group(store_id)
egen fineline = group(fineline_name)
egen brand = group(brand_name)

* size of the choice set
egen unique_prod_market = tag(market nutrition_id)
egen choice_set_size = total(unique_prod_market), by(market)
* inhstrument for size of the choice set
egen entry_week_store = min(week), by(store_id nutrition_id)
sum product
local maxJ = r(max)
forv j=1/`maxJ' {
	gen temp = entry_week_store if product==`j'
	egen temp_first_`j' = min(temp), by(store_id)
	drop temp
}
forv j=1/`maxJ' {
	gen entered_`j' = (week>=temp_first_`j')
}
forv j=1/`maxJ' {
	gen temp1b = fineline if product==`j'
	egen temp_fine`j' = max(temp1b) 
	drop temp1b
}
egen choice_set_size_instrument = rowtotal(entered_1-entered_`maxJ')
forv i=1/5 {
	forv j=1/`maxJ' {
		gen tempp_`j' = entered_`j'*(temp_fine`j'==`i')
	}
	egen fine_`i'_instrument = rowtotal(tempp_1-tempp_`maxJ')
	drop tempp*
}
drop temp*

* include heterogeneous beliefs
ren mu_cal mu_cal_homo 
ren mu_sug mu_sug_homo 
ren sigma_cal sigma_cal_homo
ren sigma_sug sigma_sug_homo

merge n:1 nutrition_id income_bins using "${Repo}/00_files/02_intermediate/04_beliefs/03_cereals_beliefs_model_income.dta"
keep if _merge==3
drop _merge

* add parent holding company
preserve
import excel "${Repo}/00_files/01_source/01_walmart/holding.xlsx", sheet("Sheet1") firstrow clear
keep product holding 
cap tempfile hold_temp
save `hold_temp', replace
restore

merge n:1 product using `hold_temp'
keep if _merge==3
drop _merge
ren holding holding_name
egen holding = group(holding_name)

drop mu_sug sigma_sug mu_cal sigma_cal
ren mu_bayes_sug mu_sug
ren mu_bayes_cal mu_cal
ren sigma_homo_cal sigma_cal 
ren sigma_homo_sug sigma_sug 

* generate labels
tempfile rf
foreach var in sug cal {
ren L`var' l_`var'
gen l_`var'_post = l_`var'*post
preserve
tab fineline, gen(fl)
duplicates drop product post, force
rforest l_`var'_post post `var'_pre fl* , type("class") iter(100) s(2019) 
predict l_hat_0 l_`var'_prob , pr
predict l_`var'_hat 
keep product post l_`var'_hat l_`var'_prob
save `rf', replace
restore
merge n:1 product post using `rf'
drop _merge
}

* include ads
merge n:1 nutrition_id post using "${Repo}/00_files/02_intermediate/05_advertising/01_cereals_tvAds.dta"
gen on_ad = _merge==3
drop if _merge==2
drop _merge
foreach var in length_rating length length_per_prod ads_per_prod total_ads {
	replace `var'=0 if `var'==.
}

* final details

gen delta = log(s)-log(s0)

gen temp = sales_final*(week<=penetration)
egen weight_sales = total(temp), by(product store individual_bins)

drop totq totqfineline totqhealthy temp* policy penetration Lsod Lfat temp sales_final_bigmarket sales_raw_bigmarket quantity_bigmarket

gen age_bins = 1

order week store store_id market product nutrition_id name brand brand_name fineline fineline_name healthy exempted s s0 s_inside s_fineline s_healthy p sug sug_pre sug_post sug_late cal cal_pre cal_post cal_late sod sod_pre sod_post sod_late fat fat_pre fat_post fat_late post l_sug l_sug_post l_sug_hat l_sug_prob l_cal l_cal_post l_cal_hat l_cal_prob mu_sug sigma_sug mu_cal sigma_cal mu_sug_homo sigma_sug_homo mu_cal_homo sigma_cal_homo inst_haus inst_dis market_size_bin market_size portion_size_bin portion_size choice_set_size_instrument

order fine_1_instrument-fine_5_instrument entered_1-entered_`maxJ', last

compress

save "${Repo}/00_files/03_data/01_marketLevel_cereals_bins.dta", replace


******************************************************************************** 
** EXPORT TO CSV
******************************************************************************** 

use "${Repo}/00_files/03_data/01_marketLevel_cereals_bins.dta", clear

drop if s==.

gen policy= floor((61-1)/8)+1         
gen penetration = floor((44-1)/8)+1
drop if week>penetration & week<=policy
rename market market_stata 
rename market_bin market_bin_stata
egen market = group(week store)
egen market_bin = group(week store individual_bins)

sort market_bin product 

order week store market market_bin product brand fineline healthy exempted s s0 s_inside s_fineline s_healthy p sug sug_pre cal cal_pre post l_sug l_sug_post l_cal l_cal_post mu_sug mu_cal sigma_sug sigma_cal  package inst_haus inst_dis l_sug_hat l_sug_prob l_cal_hat l_cal_prob choice_set_size market_size market_size_bin portion_size portion_size_bin individual_bins age_bins income_bins weight_sales delta mu_sug_homo mu_cal_homo sigma_sug_homo sigma_cal_homo name brand_name market_stata market_bin_stata sug_late cal_late inst_sug inst_corn inst_wheat inst_oat inst_rice inst_cocoa holding choice_set_size_instrument hat_p hat_fine_p_mean* hat_fine_p_p20* hat_fine_p_p80* entered_* fine_* corn wheat oat rice cocoa ads_per_prod length_per_prod length_rating
keep week store market market_bin product brand fineline healthy exempted s s0 s_inside s_fineline s_healthy p sug sug_pre cal cal_pre post l_sug l_sug_post l_cal l_cal_post mu_sug mu_cal sigma_sug sigma_cal  package inst_haus inst_dis l_sug_hat l_sug_prob l_cal_hat l_cal_prob choice_set_size market_size market_size_bin portion_size portion_size_bin individual_bins age_bins income_bins weight_sales delta mu_sug_homo mu_cal_homo sigma_sug_homo sigma_cal_homo name brand_name market_stata market_bin_stata sug_late cal_late inst_sug inst_corn inst_wheat inst_oat inst_rice inst_cocoa holding choice_set_size_instrument hat_p hat_fine_p* entered_* fine_* corn wheat oat rice cocoa ads_per_prod length_per_prod length_rating

compress

export delimited using "${Repo}/00_files/03_data/01_marketLevel_cereals_bins.csv", replace


******************************************************************************** 
** AGGREGATE TO NO BINS
******************************************************************************** 


use "${Repo}/00_files/03_data/01_marketLevel_cereals_bins.dta", clear

* redefine bins
ren individual_bins individual_bins_old

* aggregate beliefs
preserve
duplicates drop nutrition_id age_bins income_bins mu_sug sigma_sug obs_sug p_sug mu_cal sigma_cal obs_cal p_cal, force
foreach nut in sug cal {
	replace mu_`nut' = mu_`nut'*obs_`nut'
	replace p_`nut' = p_`nut'*obs_`nut'
}
collapse (sum) mu_sug obs_sug p_sug mu_cal obs_cal p_cal , by(nutrition_id)
foreach nut in sug cal {
	replace mu_`nut' = mu_`nut'/obs_`nut'
	replace p_`nut' = p_`nut'/obs_`nut'
}
gen sigma_sug = -5/invnormal((1-p_sug/100)/2)  
gen sigma_cal = -15/invnormal((1-p_cal/100)/2)
cap tempfile temp_beliefs
save `temp_beliefs', replace
restore

merge n:1 nutrition_id using `temp_beliefs'
drop _merge

* fix market size and portion_size option 1
ren market_size_bin temp
ren portion_size_bin temp2
egen unique_marketbin = tag(store_id individual_bins_old)
egen market_size_bin = total(unique_marketbin*temp), by(store_id)
egen temp3 = total(unique_marketbin*temp2*temp), by(store_id)
gen portion_size_bin = temp3/market_size_bin
drop temp* 

* redefine market_bin
drop market_bin
egen market_bin = group(market)

* size of the choice set
drop choice_set_size unique_prod_market
egen unique_prod_market = tag(market nutrition_id)
egen choice_set_size = total(unique_prod_market), by(market)

* calculate outside option
drop s0
egen inside_quantity = total(quantity), by(market_bin)
gen s0 = 1-inside_quantity/(market_size_bin*portion_size_bin)

* collapse data
collapse (first) week store store_id market product name brand brand_name fineline fineline_name healthy exempted p ///
sug sug_pre sug_post sug_late cal cal_pre cal_post cal_late sod sod_pre sod_post sod_late fat fat_pre fat_post fat_late post ///
l_sug l_sug_post l_sug_hat l_sug_prob l_cal l_cal_post l_cal_hat l_cal_prob mu_sug sigma_sug mu_cal sigma_cal inst_haus inst_dis ///
mu_sug_homo mu_cal_homo sigma_sug_homo sigma_cal_homo ///
 market_size_bin market_size portion_size_bin portion_size choice_set_size_instrument ///
 Formato Local Region mercado comuna cluster_walmart tp_cal tp_sug price_kilo price_raw s0 choice_set_size holding holding_name ///
 (sum) quantity quantity_unit sales_final sales_raw , by(nutrition_id market_bin)

* recalculate variables
egen totq = sum(quantity), by(market_bin)
gen s_inside = quantity/totq
egen totqfineline = sum(quantity), by(market_bin fineline)
gen s_fineline = quantity/totqfineline
egen totqhealthy = sum(quantity), by(market_bin healthy)
gen s_healthy = quantity/totqhealthy
gen s = quantity/(market_size_bin*portion_size_bin)
gen price_bins = (sales_final/quantity)/(650*10)
gen package = quantity/quantity_unit
gen delta = log(s)-log(s0)
gen penetration = floor((44-1)/8)+1
gen temp = sales_final*(week<=penetration)
drop penetration
egen weight_sales = total(temp), by(product store)
drop temp

gen individual_bins = 1
gen age_bins = 1
gen income_bins = 1

order week store market market_bin product nutrition_id name brand brand_name fineline fineline_name healthy exempted s s0 s_inside s_fineline s_healthy p sug sug_pre cal cal_pre post l_sug l_sug_post l_cal l_cal_post mu_sug mu_cal sigma_sug sigma_cal  package inst_haus inst_dis l_sug_hat l_sug_prob l_cal_hat l_cal_prob choice_set_size market_size market_size_bin portion_size portion_size_bin individual_bins age_bins income_bins weight_sales delta mu_sug_homo mu_cal_homo sigma_sug_homo sigma_cal_homo 

sort week store product 

compress

save "${Repo}/00_files/03_data/02_marketLevel_cereals_nobins.dta", replace


******************************************************************************** 
** EXPORT TO CSV
******************************************************************************** 

use "${Repo}/00_files/03_data/02_marketLevel_cereals_nobins.dta", clear

drop if s==.

gen policy= floor((61-1)/8)+1         
gen penetration = floor((44-1)/8)+1
drop if week>penetration & week<=policy
rename market market_stata 
rename market_bin market_bin_stata
egen market = group(week store)
gen market_bin = market

sort market product 

order week store market market_bin product brand fineline healthy exempted s s0 s_inside s_fineline s_healthy p sug sug_pre cal cal_pre post l_sug l_sug_post l_cal l_cal_post mu_sug mu_cal sigma_sug sigma_cal  package inst_haus inst_dis l_sug_hat l_sug_prob l_cal_hat l_cal_prob choice_set_size market_size market_size_bin portion_size portion_size_bin individual_bins age_bins income_bins weight_sales delta mu_sug_homo mu_cal_homo sigma_sug_homo sigma_cal_homo name brand_name 
keep week store market market_bin product brand fineline healthy exempted s s0 s_inside s_fineline s_healthy p sug sug_pre cal cal_pre post l_sug l_sug_post l_cal l_cal_post mu_sug mu_cal sigma_sug sigma_cal  package inst_haus inst_dis l_sug_hat l_sug_prob l_cal_hat l_cal_prob choice_set_size market_size market_size_bin portion_size portion_size_bin individual_bins age_bins income_bins weight_sales delta mu_sug_homo mu_cal_homo sigma_sug_homo sigma_cal_homo name brand_name  market_stata market_bin_stata


export delimited using "${Repo}/00_files/03_data/02_marketLevel_cereals_nobins.csv", replace

******************************************************************************** 
** DELETE TEMPORAL FILES
******************************************************************************** 

forv i=1/4 {
cap rm "${Repo}/00_files/03_data/temp/0`i'_carealpanel.dta"
}
